# 2 sound paradigms: 
#   - random: 
#   - oddball: there is always at least 3 standards between consecutive deviants
#   - each of them has 12.5 probability of the deviant sound (1 in 8 is deviant)

# 2 visual task loads: 
#   - low load: all letters the same
#   - high load: 6 different letters, drawn randomly without replacement

# each condition is repeated 2 times (together: 8 blocks)

# Randomizations: 
#   - blocks are randomized for every participant
#   - LL and HL don't alternate
#   - there cannot be more than 3 consecutive targets or non targets
#   - 20% of trials = targets (20% of std and 20% of dev)

from __future__ import division
from psychopy import visual
from psychopy import core, gui, data, event, parallel
from psychopy.tools.filetools import fromFile, toFile
import sounddevice as sd
import time, numpy, random, os.path, math, sys
import numpy as np
import mn_mmn4 as mn
import matplotlib.pyplot as plt
import threading

# Functions --------------------------------------------------------------------
def check4esc():
    ''' Press ESC to force stop the experiment '''
    keys = event.getKeys(keyList = ['escape'])
    if len(keys)>0 and 'escape' in keys:
        core.quit()

def trigger(trig,dur):
    target = p_port.setData(trig)
    core.wait(dur) # duration of trigger
    p_port.setData(255) # turn sound trigger off

def paus(t,pauses,block,trig = 223):
    ''' 
    Keyword arguments:
    pauses -- list of trialnumbers where there should be a pauses (list)
    t -- the current trialnumber (int)
    block -- list of blocknumbers, ex (0,0,0,1,1,1,2,2,2) (list)
    trig -- what trigger to send for pause (int)
    '''
    if t in pauses:
        if t != pauses[len(pauses)-1]:
            paus_info.setText('Short break\n%i/%i blocks done.\nHit space when ready.'% (block[t+1], len(pauses)))
            paus_info.draw()
            win.flip()
            event.waitKeys(keyList=['space'])
            trigger(trig,0.01)
            win.flip()
            core.wait(2)

def response_check(key):
    """
    Checks if a key was pressed.
    Keyword arguments:
    key -- containing either a keypress and a time or nothing (list)

    return:
    time -- nan if not pressed or time of press if pressed
    """
    if len(key) == 0:
        pressed = 0
    elif 'space' in key[0]:
        pressed = 1
    if pressed == 1:
        time = key[0][1]
    elif pressed == 0:
        time = 'nan'

    return str(time), pressed

p_port = parallel.ParallelPort(address = "0x4020")
#os.chdir('C:\Users\RAEK2031\Dropbox\Rasmus_Eklund\MMN\Data') # work computer
#os.chdir('C:\Users\SWIENSLAB03\Dropbox\Rasmus_Eklund\MMN\MMN_final\Data') # big screen
#os.chdir('C:\Users\MASZ6827\Dropbox\Rasmus_Eklund\MMN\Data') # Malinowy computer
os.chdir('C:\Users\SWIENSLAB03\Desktop\MMN4\mmn_final\Data') # desktop on lab comp

random.seed()
n_blocks = 8
n_trials = 2936 # 360 * 8 + 7 * 8

# present a dialogue to give subject nr and select program
expInfo = gui.Dlg(title="MMN4")
expInfo.addField('Subject', '01')
ok_data = expInfo.show()  # show dialog and wait for OK or Cancel
if expInfo.OK:  # or if ok_data is not None
    print('Subject: %i\n' %(int(ok_data[0])))
else:
    print('user cancelled')
    core.quit()

# Screen
win = visual.Window([1920, 1080], allowGUI = False, # [1920, 1080]
    monitor = 'testMonitor', units = 'height', color = 'gray')

# ==============================================================================
# RANDOMIZE TRIAL ORDER AND ASSIGN RESPONSES------------------------------------

# RANDOM -----------------------------------------------------------------------
# 1 - 500 Hz deviant/control
# 0 - 550 Hz standard

# 2 - 605 Hz
# 3 - 666 Hz
# 4 - 732 Hz
# 5 - 805 Hz
# 6 - 886 Hz
# 7 - 974 Hz
set = [0,1,2,3,4,5,6,7]
def random_tone_order(set, n_blocks):
    random_cond = []
    for t in range(n_blocks): 
        '''
        Returns a random sound order with some restrictions.
        
        This loop first make a vector with randomly ordered tones (trials), 
        then takes the same vector, shifts it one position to the right and 
        subtracts from the original one. If there are any "0" in the result of
        the subtraction, it randomizes a new vector until there are no "0s". 
        This makes sure that there are never two of the same sound in a row 
        within and between two sets of tones.
        '''
        tmp = [9] # shifter, dead weight to be able to shift by 1 and subtract
        set_tones = list(np.random.choice(set, 8, replace = False))
        while len(tmp) < 369:
            if 0 in (np.array(((tmp+set_tones)[len(tmp+set_tones)-9:len(tmp+set_tones)-1])-np.array(set_tones))):
                set_tones = list(np.random.choice(set, 8, replace = False))
            else:
                tmp += set_tones
                set_tones = list(np.random.choice(set, 8, replace = False))
        del tmp[[i for i, set_tones in enumerate(tmp[0:9]) if set_tones == 1][0]] # delete deviant from the first set
        del tmp[0] # delete first number in the vector: 9
        random_cond += [tmp]
    return random_cond

def prob_dist(resp_pool,set_range):
    '''
    This function calculates the probability of randomly picking each individual number (sound) from all pools.
    This function checks if any of the remaining pool numbers are in the first 5 and 3 positions of the current set.
    
    Keyword arguments:
        resp_pool -- a list of lists with conditions from 0 to 7 :[[0,0,0,...],[1,1,1,...],...]
        set_range -- the current set of conditions that we want to assign responses to
    Returns:
        prob_norm -- probability distribution for how many responses that are left in each slot of the pool
        p_test_result -- logical, true if any of the remaining pool numbers are in the first 5 and 3 positions of the current set
    '''
    prob_raw = []
    for i in range(8): # calculate proportion responses left per subpool
        prob_raw += [len(resp_pool[i])/float(9)] # should divide by nr of sounds per subpool in resp_pool

    prob_norm =[]
    for i in range(8): # normalize proportions to get probability distribution for all pools
        prob_norm += [prob_raw[i]/sum(prob_raw)]

    prob_test = []
    for i in range(8): # mark non-empty subpools with 1 and 0
        prob_test += [math.ceil(prob_norm[i])]

    prob_test_index = []
    for i, x in enumerate(prob_test): # find and enumerate non-empty subpools
        if x == 1:
            prob_test_index += [i]

    p_test_result = False

    for a in prob_test_index: # checks if any of the remaining pool numbers are in the first 5 and 3 positions of the current set
        if a in set_range[0:5]:
            for b in prob_test_index:
                if b in set_range[0:3]:
                    p_test_result = True
    
    return prob_norm, p_test_result

def assign_responses_random(random_cond):
    all_random_responses = []
    for block in range(8):
        test = True
        while test:
            resp_pool = [] # makes the response pool (list of lists with all sounds and their respective responses)
            for i in range(8):
                resp_pool.append(9 * [i])
                
            jumps = [7] + [5] * 72

            random_responses = []
            k = 0
            try: 
                for j in jumps: # jumps with 7 or 5 steps
                    if j == 5: # if jump = 5
                        tone = 99
                        p_dist, p_test = prob_dist(resp_pool,random_cond[block][k:k+j]) # this function looks if there are numbers left in the pool to fit into the set
                        if p_test: # if there are responses left to fit in the current set
                            while tone not in random_cond[block][k:k+3]: # look into first three and see if chosen sound is there
                                tone = int(np.random.choice(a = set, size = 1, replace = True, p = p_dist))# randomly chose a sound
                            tmp = [1,0] # if there are two of the selected sound in the first three
                            for i in range(j): # loop through current jump
                                if random_cond[block][k] == tone: # if the position contains the current sound
                                    random_responses += [tmp[0]] # add a response
                                    if tmp[0] == 1:
                                        del resp_pool[tone][0] # and delete a response from that sound's pool
                                    del tmp[0]
                                else: 
                                    random_responses += [0] # if the position does not contain the current sound, add a non-response
                                k += 1
                    elif j == 7: # if jump = 7
                        for zero in range(j): # add 7 non-responses
                            random_responses += [0]
                            k += 1
                    else:
                        print('Error')
                    if len(random_responses)== 367:
                        test = False
                        print('Found solution for random order for block %i.\n' %(block+1))
                        all_random_responses += random_responses
            except (IndexError,ValueError) as e:
                print (e)
                continue
    #print(all_random_responses)
    return all_random_responses

# ODDBALL ----------------------------------------------------------------------
# 1 - 500 Hz deviant/control
# 0 - 550 Hz standard

# 45 miniblock containing 360 trials.
# These numbers represent how many standards are between deviants.
# The smallest number is 4 because we want to run a loop with range 4 and insert
# a deviant the first iteration and 3 standards for the rest.
def tones_responses_oddball():
    miniblocks = [4]*2+[5]*3+[6]*5+[7]*8+[8]*9+[9]*8+[10]*5+[11]*3+[12]*2
    all_oddball_responses = []
    all_oddball_cond = []
    for block in range(8):
        test = True
        while test:
            try:
                # Randomize miniblocks and add eight sets of them with 7 zeroes before every set
                rand_miniblocks = []
                resp_dev = []

                rand_miniblocks = list(np.random.choice(miniblocks, 45, replace = False))
                resp_dev = list(np.random.choice([0]*36 + [1]*9, 45, replace = False)) # this will sum to 9

                # This makes a condition list with 1 for deviant and 0 for standard (20% deviants)
                # Parallel, a response list with 1 for response and 0 for non-responses (20% responses) ONLY FOR DEVIANTS
                oddball_cond = []
                oddball_response = []

                oddball_cond = [0]*7
                oddball_response = [0]*7
                for mb in range(45): # loop 45 miniblocks
                    for trial in range(rand_miniblocks[mb]): # loop within miniblocks (4 to 12 iterations)
                        if trial == 0: # if first iteration
                            oddball_cond += [1] # add a 1 for deviant in condition list
                            oddball_response += [resp_dev[0]] # and add number to response list (from a 20% list, this is to the deviant)
                            del resp_dev[0]
                        else:
                            oddball_cond += [0] # if not deviant, add 0 as standard
                            oddball_response += [0] # and add non-responses to the rest of standards
                
                resp_pool = [1] * 63 # responses for standards
                jumps = [7] + [5] * 72

                k = 0
                                
                for j in jumps:
                    if j == 7:
                        k += 7
                    if j == 5:
                        pool_pos = [0,1,2] # pool of positions
                        if sum(oddball_response[k:k+j]) == 0: 
                            # This if loop first looks at the five current positions in response vector. 
                            # Are there responses? if yes, then move to the next 5 positions
                            # if no, do the following:
                            
                            if sum(oddball_response[k-2:k]) == 0: 
                            # if there are no responses in the two previous positions: 
                            
                                if sum(oddball_cond[k:k+j-2]) == 1: 
                                # if there is a deviant with no target in the first three positions
                                    pos_y = [i for i,a in enumerate(oddball_cond[k:k+j-2]) if a == 1] # find its position
                                    del(pool_pos[pos_y[0]]) # and delete it from the pool of possible positions
                                    oddball_response[k+int(np.random.choice(pool_pos, 1))] = resp_pool[0] # add 1 to the standard in the response vector
                                    del(resp_pool[0])
                                    
                                elif sum(oddball_cond[k:k+j-2]) == 0:
                                # if there is no deviant within the first three
                                    oddball_response[k+int(np.random.choice(pool_pos, 1))] = resp_pool[0]
                                    del(resp_pool[0])
                                
                                else:
                                    print('ERROR! more than 1 deviant within three positions') 
                                    
                            elif oddball_response[k-1] == 1: 
                            #if there is a response in the first previous position (closest previous)
                                oddball_response[k+2] = resp_pool[0]
                                del(resp_pool[0])
                                
                            elif oddball_response[k-2] == 1: 
                            # if there is a response in the second previous position (not closest previous)
                                del(pool_pos[0]) # we cannot put the response in the position 0, so delete it
                                
                                if oddball_cond[k+2] == 1: 
                                # if there is a deviant with no target in the position 2 (deviants: 1, 0, 0(k), 0, 1(dev in the third pos))
                                    oddball_response[k+1] = resp_pool[0] # add 1 to the standard in the response vector
                                    del(resp_pool[0])
                                    
                                elif oddball_cond[k+2] == 0: 
                                # if there is no deviant with no target in the position 2
                                    oddball_response[k+int(np.random.choice(pool_pos, 1))] = resp_pool[0]
                                    del(resp_pool[0])
                            else:
                                print('ERROR! cannot find a solution')
                                
                        k += 5
                    if sum(oddball_response)== 72 and k == 367:
                        test = False
                        print('Found solution for oddball order.\n')
            except (IndexError,ValueError) as e:
                print (e)
                continue
        all_oddball_responses += oddball_response
        all_oddball_cond += oddball_cond
    return all_oddball_cond, all_oddball_responses
# ==============================================================================
# CREATE SOUNDS ----------------------------------------------------------------

sd.default.device = "ASIO Fireface USB"
print 'Sound device ------------------------------------------------------------'
print sd.query_devices()#device = "ASIO Fireface USB")
print '-------------------------------------------------------------------------'

# Set the gain and sampling frequency (fs)
gain = -20.45
fs = 44100

frequencies = [500, 550, 605, 666, 732, 805, 886, 974]
tones       = [0]*8
# Make them equally loud
gains = [gain, gain+0.1, gain-0.1, gain-0.65, gain-1.45, gain-2.65, gain-3.8, gain-3.8] 

for t in range(len(frequencies)):
    tones[t] = mn.set_gain(mn.fade(mn.create_sinusoid(
        freq = frequencies[t], phase = 0, fs = fs, dur = 0.1),441),gains[t]) # 100 ms, 10 ms fade in/out

f_500 = np.transpose(np.array([tones[0],tones[0]])) # deviant, control
f_550 = np.transpose(np.array([tones[1],tones[1]])) # standard
f_605 = np.transpose(np.array([tones[2],tones[2]]))
f_666 = np.transpose(np.array([tones[3],tones[3]]))
f_732 = np.transpose(np.array([tones[4],tones[4]]))
f_805 = np.transpose(np.array([tones[5],tones[5]]))
f_886 = np.transpose(np.array([tones[6],tones[6]]))
f_974 = np.transpose(np.array([tones[7],tones[7]]))

# ==============================================================================
# CREATE VISUALS ---------------------------------------------------------------

intro_text = visual.TextStim(
    win, color = 'white', height = 0.03, pos = (0, 0), text = 'Press space to start the experiment!')
end_text = visual.TextStim(
    win, color = 'white', height = 0.03, pos = (0, 0), text = 'End of experiment!\n\nThank you for participating!')
paus_info = visual.TextStim(
    win, color = 'white', height = 0.03, pos = (0, 0), text = 'paus')
processing = visual.TextStim(
    win, color = 'white', height = 0.03, pos = (0, 0), text = 'Setting up experiment.')
target_cross_info = visual.TextStim(
    win, color = 'white', height = 0.03, pos = (0, 0), text = 'Press space when you see these crosses.')
# Assign crosses to response and nonresponse trials
def cross(responses,targets,nontargets):
    '''
    Assigns cross conditions (colors * 5 and orientations * 2) to response and 
    nonresponse trials for high and low load.
    
    Keyword arguments:
    reponses -- list of 1 and 0 for target and nontarger respectively
    targets -- list target condtions (see above)
    nontargets -- list of nontarget conditions
    
    Return:
    cross_cond -- a list of conditions (0 to 9)
    '''
    cross_cond = []
    pool = [[[],[],[],[],[],[],[],[]],[[],[],[],[],[],[],[],[]],[]]
    pool[2] += list(np.random.choice(nontargets, 7*8, replace = True))
    
    blocks_counter = []
    for i in range(8):
        blocks_counter += [i]*367
    
    seven = []
    for i in range(8):
        seven += [1]*7 + [0]*360

    for b in range(n_blocks):
        for c in range(36): # creates target and nontarget codition pools. 576/8 = 72 (targets per block). 2304/8 = 288
            pool[0][b] += targets # target pool
            pool[1][b] += nontargets # nontarget pool
    counter = 0
    for i in responses:
        if seven[counter] == 1: # if its the first 7
            index = np.random.choice(range(len(pool[2])), 1) # randomly select a position from the pool
            cross_cond += [pool[2][index[0]]] # assign randomized index
            del pool[2][index[0]] # delete the selected one from the pool
        if seven[counter] == 0: # if its not first 7
            if i == 1: # randomly assigns a target cross to a response trial
                index = np.random.choice(range(len(pool[0][blocks_counter[counter]])), 1) # randomly select a position from the pool
                cross_cond += [pool[0][blocks_counter[counter]][index[0]]] # assign randomized index
                del pool[0][blocks_counter[counter]][index[0]] # delete the selected one from the pool
            elif i == 0: # randomly assigns a target cross to a nonresponse trial
                index = np.random.choice(range(len(pool[1][blocks_counter[counter]])), 1)
                cross_cond += [pool[1][blocks_counter[counter]][index[0]]]
                del pool[1][blocks_counter[counter]][index[0]]
        counter += 1
    return cross_cond

# Shows targets for this block
def show_target_crosses(pauses, t, trial_paradigm, targethi, targetlo):
    '''
    Shows what crosses are targets for this block.
    
    Keyword arguments:
    pauses -- list of trialnumbers where there should be a pauses (list)
    t -- the current trialnumber (int) 
    '''
    start_block = [i - 366 for i in pauses]
    if t in start_block:
        lohi = targetlo,targetlo,targethi,targethi
        #set stimuli according to condition
        t1_standing = visual.Rect(win=win, name='up_cross_hor', width=(dimentions[1]), 
            height=(dimentions[0]), ori=0, 
            pos=(-0.2, 0-0.2), lineWidth=1, 
            lineColor=colors[lohi[trial_paradigm][0]], lineColorSpace='rgb', 
            fillColor=colors[lohi[trial_paradigm][0]], fillColorSpace='rgb', opacity=1, depth=0.0, interpolate=True)
        
        t1_laying = visual.Rect(win=win, name='up_cross_hor', width=(dimentions[0]), 
            height=(dimentions[1]), ori=0, 
            pos=(-0.2, position[lohi[trial_paradigm][0]]-0.2), lineWidth=1, 
            lineColor=colors[lohi[trial_paradigm][0]], lineColorSpace='rgb', 
            fillColor=colors[lohi[trial_paradigm][0]], fillColorSpace='rgb', opacity=1, depth=0.0, interpolate=True)
        
        t2_standing = visual.Rect(win=win, name='up_cross_hor', width=(dimentions[1]), 
            height=(dimentions[0]), ori=0, 
            pos=(0.2, 0-0.2), lineWidth=1, 
            lineColor=colors[lohi[trial_paradigm][1]], lineColorSpace='rgb', 
            fillColor=colors[lohi[trial_paradigm][1]], fillColorSpace='rgb', opacity=1, depth=0.0, interpolate=True)
        
        t2_laying = visual.Rect(win=win, name='up_cross_hor', width=(dimentions[0]), 
            height=(dimentions[1]), ori=0, 
            pos=(0.2, position[lohi[trial_paradigm][1]]-0.2), lineWidth=1, 
            lineColor=colors[lohi[trial_paradigm][1]], lineColorSpace='rgb', 
            fillColor=colors[lohi[trial_paradigm][1]], fillColorSpace='rgb', opacity=1, depth=0.0, interpolate=True)
            
        t1_standing.draw() # show vertical bar
        t1_laying.draw() # show horizontal bar
        t2_standing.draw() # show vertical bar
        t2_laying.draw() # show horizontal bar
        target_cross_info.draw()
        win.flip()
        event.waitKeys(keyList=['space'])
        win.flip()
        core.wait(2)

#photo diode stimulator
whiteOn = visual.Rect(
    win=win, name='WhiteOn',
    width=(0.025, 0.025)[0], height=(0.025, 0.025)[1],
    ori=0, pos=(0.88, 0.49),
    lineWidth=1, lineColor=[1.000,1.000,1.000], lineColorSpace='rgb',
    fillColor=[1.000,1.000,1.000], fillColorSpace='rgb',
    opacity=1, depth=0.0, interpolate=True)

# Make a text file to save data ------------------------------------------------
fileName = 'fp_' + ok_data[0]
dataFile = open(fileName+'.txt', 'w')
dataFile.write('fp\tblock_nr\ttrial_nr\tcondition\tsound\ttrigger\ttarget\tcross_cond\ttrial_start\tresponse_time\n')

# ==============================================================================
# MAKE BLOCKS, PAUSES AND CONDITIONS
processing.draw()
win.flip()

blocks = []
pauses = []
for i in range(n_blocks):
    pauses.append([])
    pauses[i] = int(n_trials/n_blocks * i + (n_trials/n_blocks - 1))
    blocks += [i] * int(n_trials/n_blocks)

# tone * load conditions -------------------------------------------------------
low_oddball_cond, low_oddball_response =    tones_responses_oddball()
hi_oddball_cond , hi_oddball_response =     tones_responses_oddball()
low_random_cond =       random_tone_order(set,n_blocks)
low_random_response =   assign_responses_random(low_random_cond)
hi_random_cond =        random_tone_order(set,n_blocks)
hi_random_response =    assign_responses_random(hi_random_cond)

# untangle random_conds
temp = []
temp1 = []
for i in range(8):
    temp += low_random_cond[i]
    temp1 += hi_random_cond[i]
low_random_cond = temp
hi_random_cond = temp1
# Cross conditions:

# up    red     0   low load
# down  red     1   low load
# up    blue    2
# down  blue    3
# up    yellow  4   hi load
# down  yellow  5
# up    purple  6
# down  purple  7
# up    green   8
# down  green   9   hi load

hi_targets = [4,9]
low_targets = [0,1]
hi_nontargets = [0,1,2,3,5,6,7,8]
low_nontargets = [2,3,4,5,6,7,8,9]

# Assign crosses to response and nonresponse trials ----------------------------
hi_oddball_crosses =    cross(hi_oddball_response, hi_targets, hi_nontargets)
low_oddball_crosses =   cross(low_oddball_response, low_targets, low_nontargets)
hi_random_crosses =     cross(hi_random_response, hi_targets, hi_nontargets)
low_random_crosses =    cross(low_random_response, low_targets, low_nontargets)

# Triggers: 
# load    paradigm  tone      trigger     code          trig + trgt     code + trgt
# low     oddball   dev       254         x o o o o o   238             x o o o x o 
# low     oddball   std       253         o x o o o o   237             o x o o x o 
# low     random    control   251         o o x o o o   235             o o x o x o 
# low     random    other     247         o o o x o o   231             o o o x x o 
# hi      oddball   dev       252         x x o o o o   236             x x o o x o 
# hi      oddball   std       249         o x x o o o   233             o x x o x o 
# hi      random    control   243         o o x x o o   227             o o x x x o 
# hi      random    other     246         x o o x o o   230             x o o x x o 
# end of the block            192         x x x x x x
# space press                 223         o o o o o x

all_paradigms       = [low_oddball_cond, low_random_cond, hi_oddball_cond, hi_random_cond]
all_sounds          = [f_550,f_500,f_605,f_666,f_732,f_805,f_886,f_974]
all_sounds_names    = ['550','500','605','666','732','805','886','974']
trig_list           = [[253,254],[247,251,247,247,247,247,247,247],[249,252],[246,243,246,246,246,246,246,246]]
trig_list_targets   = [[237,238],[231,235,231,231,231,231,231,231],[233,236],[230,227,230,230,230,230,230,230]]
all_responses       = [low_oddball_response,low_random_response, hi_oddball_response, hi_random_response]
all_crosses         = [low_oddball_crosses, low_random_crosses, hi_oddball_crosses, hi_random_crosses]

# block_paradigm:
# 0 - oddball low load OL
# 1 - random low load RL
# 2 - oddball high load OH
# 3 - random high load RH

np.random.seed(int(ok_data[0]))
block_paradigm = []
for t in range(2):
    block_paradigm += list(np.random.choice([0,1,2,3], 4, replace = False))

np.random.seed()

trial_paradigm = [] # Contains the paradigm on every trial (1 to 4)
for i in range(n_blocks):
    trial_paradigm += [block_paradigm[i]]*367 # 360 trials per condition(block) plus 7 standards in beginning of each one

#first seven
first_seven = []
for i in range(8):
    first_seven += [1]*7 + [0]*360

#           red                 blue              yellow            purple             green
colors = [1,-1,-1],[1,-1,-1],[-1,-1,1],[-1,-1,1],[1,1,-1],[1,1,-1],[1,-1,1],[1,-1,1],[-1,1,-1],[-1,1,-1]
# ver, hor
dimentions = [0.100,0.010]
position = [0.0255,-0.0255]*5

globalClock = core.Clock()  # to keep track of time
respClock = core.Clock()
# ==============================================================================
# Experimental procedure -------------------------------------------------------
trigger(255,0.01)
intro_text.draw()
win.flip()
event.waitKeys(keyList=['space'])
trigger(192,0.01) # first block start trigger
win.flip()

# Trial loop
for t in range(n_trials): # n_trials is the total amount of trials
    show_target_crosses(pauses, t, trial_paradigm[t], hi_targets, low_targets)# show target for next block
    l_trial_start = globalClock.getTime()
    check4esc() # check for esc
    #set stimuli according to condition
    standing = visual.Rect(win=win, name='up_cross_hor', width=(dimentions[1]), 
        height=(dimentions[0]), ori=0, pos=(0, 0), lineWidth=1, 
        lineColor=colors[all_crosses[trial_paradigm[t]][t]], 
        lineColorSpace='rgb', fillColor=colors[all_crosses[trial_paradigm[t]][t]], 
        fillColorSpace='rgb', opacity=1, depth=0.0, interpolate=True)
    laying = visual.Rect(win=win, name='up_cross_hor', width=(dimentions[0]), 
        height=(dimentions[1]), ori=0, pos=(0, position[all_crosses[trial_paradigm[t]][t]]), lineWidth=1, 
        lineColor=colors[all_crosses[trial_paradigm[t]][t]], 
        lineColorSpace='rgb', fillColor=colors[all_crosses[trial_paradigm[t]][t]], 
        fillColorSpace='rgb', opacity=1, depth=0.0, interpolate=True)

    # time between trigger and sound should be around 410 ms
    threading.Thread(target = sd.play(all_sounds[all_paradigms[trial_paradigm[t]][t]], fs, blocking = False)).start() # Play sound for current trial on a separate thread

    if first_seven[t] == 0:
        if all_responses[trial_paradigm[t]][t] == 0:
            trigger(trig_list[trial_paradigm[t]][all_paradigms[trial_paradigm[t]][t]],0.01) # send sound trigger    
        elif all_responses[trial_paradigm[t]][t] == 1: 
            trigger(trig_list_targets[trial_paradigm[t]][all_paradigms[trial_paradigm[t]][t]],0.01)
    core.wait(0.055, hogCPUperiod = 0.055) # adjust diode to sound delay
    standing.draw() # vertical bar
    laying.draw() # horizontal bar
    whiteOn.draw() # square
    win.flip() # show cross and white
    
    #key = []
    #key = event.getKeys(keyList = ['space'], timeStamped = globalClock) # if this is not commented out then the responses don't get recorded 

    core.wait(0.1) # show cross 100 ms
    win.flip() # turn visual stuff off
    core.wait(0.032, hogCPUperiod = 0.032) #(0.032, hogCPUperiod = 0.032) # adjust ITI
    
    l_fp = int(ok_data[0])
    l_block_nr = blocks[t]+1
    l_trial_nr = (range(367)*n_blocks)[t]+1
    l_condition = trial_paradigm[t]
    l_sound = all_sounds_names[all_paradigms[trial_paradigm[t]][t]]
    if first_seven[t] == 0:
        if all_responses[trial_paradigm[t]][t] == 0:
            l_trigger = trig_list[trial_paradigm[t]][all_paradigms[trial_paradigm[t]][t]] # send sound trigger    
        elif all_responses[trial_paradigm[t]][t] == 1: 
            l_trigger = trig_list_targets[trial_paradigm[t]][all_paradigms[trial_paradigm[t]][t]]
    elif first_seven[t] == 1: 
        l_trigger = 999
    l_target = all_responses[trial_paradigm[t]][t]
    l_cross_condition = all_crosses[trial_paradigm[t]][t]
    
    key = event.getKeys(keyList = ['space'], timeStamped = globalClock)
    l_response_time = response_check(key)[0]
    
    # Save data to file
    #'fp\tblock_nr\ttrial_nr\tcondition\tsound\ttrigger\ttarget\tcross_cond\ttrial_start\tresponse_time\n'
    dataFile.write('%i\t%i\t%i\t%i\t%s\t%i\t%i\t%i\t%f\t%s\n' %(
        l_fp, l_block_nr, l_trial_nr, l_condition, l_sound, l_trigger, 
        l_target, l_cross_condition, l_trial_start, l_response_time))
    paus(t, pauses, blocks, trig = 192) # check for pauses
    #print('Response time: %s'%(l_response_time))
    #print('End trial time: %f\n\n' %(globalClock.getTime()))

dataFile.close()
end_text.draw()
win.flip()
event.waitKeys(keyList=['space'])